

?-ensure_loaded(lib_compatibility_for_swi).


remove_doubles_from_sorted_list([],[]).
remove_doubles_from_sorted_list([X],[X]):-!.
remove_doubles_from_sorted_list([X,X|Rest], Singletons):-
   !,
   remove_doubles_from_sorted_list([X|Rest], Singletons).
remove_doubles_from_sorted_list([X|Rest],[X|Singletons]):-
   remove_doubles_from_sorted_list(Rest,Singletons).

remove_doubles([],[]).
remove_doubles([X|Rest],Erg):-
   member(X,Rest),
   !,
   remove_doubles(Rest,Erg).
remove_doubles([X|Rest],[X|Erg]):-
   remove_doubles(Rest,Erg).



quicksort([],[]).
quicksort([X],[X]):-!.
quicksort([Median|Rest],Sorted):-
   findall(Kleiner, (member(Kleiner,Rest),Kleiner=<Median), Links),
   findall(Groesser, (member(Groesser,Rest),Groesser>=Median), Rechts),
   quicksort(Links,LinksSorted),
   quicksort(Rechts,RechtsSorted),
   append(LinksSorted,[Median|RechtsSorted],Sorted).

mergesort_merge([],X,X):-!.
mergesort_merge(X,[],X):-!.
mergesort_merge([A|R],[B|L],[A|RekResult]):-
   A =< B, !,
   mergesort_merge(R,[B|L],RekResult).
mergesort_merge(R,[B|L],[B|RekResult]):-
   mergesort_merge(R,L,RekResult). /* B < A is certain because of the red cut in the first clause */

mergesort([],[]):-!.
mergesort([X],[X]):-!.
mergesort(Liste, Sortiert):-
   length(Liste,Laenge),
   findall(Element, Position^(member(Element,Position,Liste),Position < Laenge//2), Links ),     /* linke Teilliste bestimmen */
   mergesort(Links, LinksSortiert),                                                        /* rekursion auf linker Teilliste */
   findall(Element, Position^(member(Element,Position,Liste),Position >= Laenge//2), Rechts ),   /* rechte Teilliste bestimmen */
   mergesort(Rechts, RechtsSortiert),                                                      /* rekursion auf rechter Teilliste */
   mergesort_merge(LinksSortiert,RechtsSortiert,Sortiert).                                           /* mergen */



member_sorted_sets(Element, [Element|_]).
member_sorted_sets(Element, [First|Rest]):-
   First < Element,
   member_sorted_sets(Element,Rest).



make_sorted_set(List,SortedSet):-
   mergesort(List,SortedList), /* since quicksort also needs twice the RAM of the list in prolog, mergesort has no
                                  disadvantage against quicksort, but is in O(n*log(n)), while quicksort is in O(n^2) */
   remove_doubles_from_sorted_list(SortedList,SortedSet).
make_set(List,Set):-
   remove_doubles(List,Set).



subset_sorted_sets(A,B):- /* A is subset of B */
   set_intersection_sorted_sets(A,B,A).

subset(A,B):-
   forall(member(X,A),member(X,B)).



superset_sorted_sets(A,B):-
   subset_sorted_sets(B,A).

superset(A,B):-
   subset(B,A).



set_intersection_sorted_sets([],_,[]):-!.
set_intersection_sorted_sets(_,[],[]):-!.
set_intersection_sorted_sets([First1|Rest1],[First2|Rest2],Result):-
   First1 < First2,
   !,
   set_intersection_sorted_sets(Rest1,[First2|Rest2],Result).
set_intersection_sorted_sets([First1|Rest1],[First2|Rest2],Result):-
   First1 > First2,
   !,
   set_intersection_sorted_sets([First1|Rest1],Rest2,Result).
set_intersection_sorted_sets([First1|Rest1],[First1|Rest2],[First1|Result]):-
   set_intersection_sorted_sets(Rest1,Rest2,Result).

set_intersection([],_,[]).
set_intersection([X|Set1],Set2,[X|Intersection]):-
   member(X,Set2),
   !,
   set_intersection(Set1,Set2,Intersection).
set_intersection([_|Set1],Set2,Intersection):-
   set_intersection(Set1,Set2,Intersection).



set_union_sorted_sets([],SortedSet2,SortedSet2):-!.
set_union_sorted_sets(SortedSet1,[],SortedSet1):-!.
set_union_sorted_sets([First1|Rest1],[First2|Rest2],[First1|Result]):-
   First1 < First2,
   !,
   set_union_sorted_sets(Rest1,[First2|Rest2],Result).
set_union_sorted_sets([First1|Rest1],[First2|Rest2],[First2|Result]):-
   First1 > First2,
   !,
   set_union_sorted_sets([First1|Rest1],Rest2,Result).
set_union_sorted_sets([First1|Rest1],[First1|Rest2],[First1|Result]):-
   set_union_sorted_sets(Rest1,Rest2,Result).

set_union([],Set2,Set2).
set_union([First1|Set1],Set2,[First1|Result]):-
   not(member(First1,Set2)),
   !,
   set_union(Set1,Set2,Result).
set_union([_|Set1],Set2,Result):-
   set_union(Set1,Set2,Result).



set_subtraction_sorted_sets([],_,[]):-!.
set_subtraction_sorted_sets(Set1,[],Set1):-!.
set_subtraction_sorted_sets([First1|Rest1],[First2|Rest2],[First1|Result]):-
   First1 < First2,
   !,
   set_subtraction_sorted_sets(Rest1,[First2|Rest2],Result).
set_subtraction_sorted_sets([First1|Rest1],[First2|Rest2],Result):-
   First1 > First2,
   !,
   set_subtraction_sorted_sets([First1|Rest1],Rest2,Result).
set_subtraction_sorted_sets([First1|Rest1],[First1|Rest2],Result):-
   set_subtraction_sorted_sets(Rest1,Rest2,Result),
   !.

set_subtraction([],_,[]):-!.
set_subtraction([First1|Rest1],Set2,[First1|Result]):-
   not(member(First1,Set2)),
   !,
   set_subtraction(Rest1,Set2,Result).
set_subtraction([_|Rest1],Set2,Result):-
   set_subtraction(Rest1,Set2,Result).







