
load_libraries:-
  ensure_loaded(lib_compatibility_for_swi),
  ensure_loaded(lib_pseudo_imperative),
  ensure_loaded(lib_sets),
  ensure_loaded(converter_epsilon_nfa_to_nfa),
  ensure_loaded(converter_nfa_to_dfa),
  ensure_loaded(converter_regexp_to_epsilon_nfa),
  ensure_loaded(exporter_c),
  ensure_loaded(exporter_latex),
  ensure_loaded('/usr/lib/pl-5.4.6/library/quintus'),
  ensure_loaded(optimize_dfa).



transition(regexp,epsilon_nfa,REGEXP,EPSILON_NFA,regexp_to_epsilon_nfa(REGEXP,EPSILON_NFA)).

transition(epsilon_nfa,nfa,EPSILON_NFA,NFA,epsilon_nfa_to_nfa(EPSILON_NFA,NFA)).
transition(dfa,optimized_dfa,DFA,OPTIMIZED_DFA,optimize_dfa(DFA,OPTIMIZED_DFA)).
transition(optimized_dfa,dfa,OPTIMIZED_DFA,DFA,DFA=OPTIMIZED_DFA).
transition(nfa,epsilon_nfa,nfa(Q,Sigma,Delta,S,F),EPSILON_NFA,EPSILON_NFA=epsilon_nfa(Q,Sigma,Delta,[],S,F)).
transition(dfa,epsilon_nfa,dfa(Q,Sigma,Delta,S,F),EPSILON_NFA,EPSILON_NFA=epsilon_nfa(Q,Sigma,Delta,[],S,F)).
transition(epsilon_nfa,latex,EPSILON_NFA,_,(epsilon_nfa_to_latex(EPSILON_NFA),fail)).
transition(nfa,c,NFA,_,(nfa_to_c(NFA),fail)).
transition(dfa,c,DFA,_,(dfa_to_c(DFA),fail)).
transition(nfa,dfa,NFA,DFA,nfa_to_dfa(NFA,DFA)).
transition(dfa,nfa,dfa(Q,Sigma,Delta,S,F),NFA,NFA=nfa(Q,Sigma,Delta,S,F)).



path(InputType,InputType,_,Input,Input):-!.
path(InputType,OutputType,TempPath,Input,Output):-
   transition(IntermediateType, OutputType, IntermediateOutput, Output, TransitionFunction),
   not(member(IntermediateType,TempPath)),
   path(InputType, IntermediateType, [IntermediateType|TempPath], Input, IntermediateOutput),
   !,
   TransitionFunction.



patsy(Input, [InputType, OutputType], Output):-
   !,
   path(InputType,OutputType,[OutputType],Input, Output).
patsy(Input, [InputType, IntermediateType | Rest], Output):-
   patsy(Input, [InputType,IntermediateType], IntermediateResult),
   patsy(IntermediateResult, [IntermediateType|Rest], Output).


?-load_libraries.


main:-
   unix(argv(Argv)), /* this is NOT platform dependant! */
   append(_,[--,InputAtom,TypePathAtom],Argv),
   term_to_atom(Input,InputAtom),
   term_to_atom(TypePath,TypePathAtom),
   patsy(Input, TypePath, Output),
   write(Output),nl,fail.

main:-
   unix(argv(Argv)),
   not((  append(_, [--,_,_],Argv)  )), /* this is NOT platform dependant! */
   nl,
   write('Prolog AutomataTheory SYstem (PATSY)      written 2007 by Viktor "AlgorithMan" Engelmann'),nl,
   write('                                                                      www.AlgorithMan.de'),nl,
   write('              released under the terms of the GNU General Public License (GPL) version 2'),nl,
   write('                                                  see GPLv2.txt for further Informations'),nl,nl,

   write('PATSY USAGE: patsy -- "Input" "[InputType, IntermediateTypes, ..., OutputType]"'),nl,
   write('   where the types can be:'),nl,
   write('      epsilon_nfa     for nondeterministic finite automata containing epsilon transitions'),nl,
   write('      nfa             for nondeterministic finite automata'),nl,
   write('      dfa             for deterministic finite automata'),nl,
   write('      optimized_dfa   for optimized deterministic finite automata (should not be used as InputType)'),nl,
   write('      regexp          for regular expressions (only as InputType)'),nl,
   write('      c               for export to a sourcecode for the C programming language (only as OutputType)'),nl,
   write('      latex           for export to a LaTeX sourcecode (only as OutputType)'),nl,
   write('                      requires the gastex package for LaTeX. PATSY was tested with GasTex 2.7'),nl,nl,

   write('   examples: patsy -- (a*b)^_ [regexp,optimized_dfa,latex]'),nl,
   write('             patsy -- "nfa([0,1,2],[a,b],[[0,a,0],[0,b,0],[0,a,1],[1,a,2]],0,[2])" "[nfa,latex]"'),nl,
   write('             patsy -- "nfa([0,1,2],[a,b],[[0,a,0],[0,b,0],[0,a,1],[1,a,2]],0,[2])" "[nfa,optimized_dfa,c]"'),nl,nl,

   write('   please consult the manual for informations on how "Input" looks'),nl,
   halt.

main:-
   halt.